#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sqlite3
import time


class Extract:

	def __init__( self , DEBUG ):
		self.stats = {
			"Pubs" : 0,
			"Pubs+Aid" : 0,
			"nodes" : 0,
			"edges" : 0,
			"times" : []
			# "cliques" : len(cliques),
		}
		self.DEBUG = DEBUG


	def get_time_s( self , start ):
		return "{0:.2f}".format((time.time() -  start))


	def calc_ACM_OCCs ( self , dbfile , t = [ "2016-01" , "2016-07"] ):

		if self.DEBUG: print( time.strftime("%H:%M:%S") , "calc_ACM.sql_OCCs" )

		self.stats["times"].append( [ time.strftime("%H:%M:%S") , "calc_ACM.sql_OCCs" ] )
		start = time.time()

		connDBLP=sqlite3.connect( dbfile )
		connDBLP.row_factory = sqlite3.Row# Magic line!
		cursorDBLP=connDBLP.cursor()

		query = 'SELECT id, date, authors FROM publication '
		query += 'WHERE date >= "'+t[0]+'" and date <= "'+t[1]+'" AND authors != "" AND ( substr(authors,1,1)!="-")'
		if self.DEBUG: 
			print(query)
			print("")

		rawdata = {}
		FreqD = {}

		cursorDBLP.execute(query)
		rows = cursorDBLP.fetchall()
		P = 0

		for pub in rows:
			P += 1
			authors = pub["authors"].split( "|" )
			authors_ids = []
			for a in authors:
				nID = int(a)
				authors_ids.append( nID )
				if nID not in FreqD:
					FreqD[ nID ] = 0
				FreqD[ nID ] += 1

			rawdata[pub["id"]] = authors_ids 

		connDBLP.close()
		self.stats["times"].append( [ self.get_time_s( start ) , "sqlite extr + elems-OCCs" ] )
		self.stats["Pubs"] = P
		self.stats["Pubs+Aid"] = P

		return rawdata , FreqD


	def extract_publications( self , dbfile , t = [ "2016-01" , "2016-07"] ):

		P = []
		connDBLP=sqlite3.connect( dbfile )
		connDBLP.row_factory = sqlite3.Row# Magic line!
		cursorDBLP=connDBLP.cursor()

		query = 'SELECT * FROM publication '
		query += 'WHERE date >= "'+t[0]+'" and date <= "'+t[1]+'"'

		cursorDBLP.execute(query)
		rows = cursorDBLP.fetchall()

		for pub in rows:
			p = {}
			for k in pub.keys():
				if pub[k] != None:
					p[k] = pub[k]
			P.append( p )


		connDBLP.close()
		return P


	def extract_sources( self , dbfile ):

		S = []
		connDBLP=sqlite3.connect( dbfile )
		connDBLP.row_factory = sqlite3.Row# Magic line!
		cursorDBLP=connDBLP.cursor()

		query = 'SELECT id,type,name FROM source '

		cursorDBLP.execute(query)
		rows = cursorDBLP.fetchall()

		for pub in rows:
			p = {}
			for k in pub.keys():
				if pub[k] != None:
					p[k] = pub[k]
			S.append( p )

		connDBLP.close()
		return S


	def extract_elements( self , dbfile , table ):

		D = {}
		connDBLP=sqlite3.connect( dbfile )
		connDBLP.row_factory = sqlite3.Row# Magic line!
		cursorDBLP=connDBLP.cursor()

		query = 'SELECT * FROM '+table

		cursorDBLP.execute(query)
		rows = cursorDBLP.fetchall()

		for pub in rows:
			p = {}
			for k in pub.keys():
				if pub[k] != None:
					p[k] = pub[k]
			D[ int(p["id"]) ] = p

		connDBLP.close()
		return D


	# def calc_ACMsqlite_FrequentItemsets ( self , dbfile , t = [ "2016-01" , "2016-07"] ):
	# 	# print( time.strftime("%H:%M:%S") , "calc_DBLP_OCCs" )
	# 	self.stats["times"].append( [ time.strftime("%H:%M:%S") , "calc_DBLP_OCCs" ] )
	# 	start = time.time()

	# 	connDBLP=sqlite3.connect( dbfile )
	# 	connDBLP.row_factory = sqlite3.Row# Magic line!
	# 	cursorDBLP=connDBLP.cursor()

	# 	query = 'SELECT id, date, authors FROM publication '
	# 	query += 'WHERE date >= "'+t[0]+'" and date <= "'+t[1]+'" AND authors != "" AND ( substr(authors,1,1)!="-")'
	# 	print(query)
	# 	print("")
	# 	cursorDBLP.execute(query)
	# 	rows = cursorDBLP.fetchall()
	# 	P = 0


	# 	A2Pubs = {}
	# 	A2Collabs = {}
	# 	# = = = [ Indexes between authors and pubs ] = = = #
	# 	# Per each pub p:
	# 	#   1. A2Collabs: Index between author_i and collaboration_p.
	# 	#       e.g: A2Collabs[ author_i ] append  collaboration_p
	# 	#   2. A2Pubs	: Index between collaboration_p AND publication.
	# 	#       e.g: A2Pubs[ collaboration_p ] append  p
	# 	for pub in rows:
	# 		authors = pub["authors"].split( "|" )
	# 		if len(authors)>1:
	# 			P += 1
	# 			# print ( ",".join( authors ) )
	# 			authors = [int(i) for i in authors]
	# 			authors_str = "_".join( map(str, sorted(authors) ) )

	# 			for a in authors:
	# 				if a not in A2Collabs:
	# 					A2Collabs[a] = []
	# 				A2Collabs[a].append( authors_str )

	# 			if authors_str not in A2Pubs:
	# 				A2Pubs[authors_str] = []
	# 			A2Pubs[authors_str].append( pub["id"] )
	# 			# print( pub["id"] , "|" , pub["date"] , "|" , pub["authors"]  )
	# 			# print("")
	# 	# = = = [ / Indexes between authors and pubs ] = = = #


	# 	Cores = {}
	# 	# = = = [ Indexes between authors_intersections and collaborations ] = = = #
	# 	# Per each collaboration_p with at least 2 pubs:
	# 	#   -  collaboration_p
	# 	#   -  Keys[]  <-- all possible collaborations of each author in collaboration_p
	# 	#   1. do intersection between collaboration_p and Keys[]
	# 	#   2. Cores	: Index between intersections (>1) and  { collaboration_p , Keys[] } .
	# 	for i in A2Pubs:
	# 		if len(A2Pubs[i])>1:
	# 			A_set_i = map(int, i.split("_") )
	# 			for single_author in A_set_i:
	# 				if single_author in A2Collabs:
	# 					Keys = A2Collabs[single_author] 
	# 					for j in Keys:
	# 						if i!=j:
	# 							if str(single_author) in j:
	# 								authors_i = i.split( "_" )
	# 								authors_i = [int(i) for i in authors_i]
	# 								authors_j = j.split( "_" )
	# 								authors_j = [int(i) for i in authors_j]
	# 								# print( i ,"AND", j  )
	# 								# print( authors_i )
	# 								# print( authors_j )
	# 								# print("")
	# 								get_ = Utils()
	# 								intersection = get_.intersect( authors_i , authors_j )
	# 								if len(intersection)>1:
	# 									intersection = "_".join( map( str , sorted(intersection) ) )
	# 									if intersection not in Cores:
	# 										Cores[ intersection ] = {}
	# 									Cores[intersection][ i ] = True
	# 									Cores[intersection][ j ] = True
	# 	# = = = [ / Indexes between authors_intersections and collaborations ] = = = #
		

	# 	P2Teams = {}
	# 	# = = = [ P2Teams: group Cores by common publications ] = = = #
	# 	for c in Cores:
	# 		# print( c )
	# 		for collab in Cores[c]:
	# 			# print( "\t" , collab )
	# 			# print( "\t\t" , A2Pubs[collab] )
	# 			for p in A2Pubs[collab]:
	# 				if p not in P2Teams:
	# 					P2Teams[p] = {}
	# 				P2Teams[p][c] = True
	# 		# print( "" )
	# 	# = = = [ / P2Teams: group Cores by common publications ] = = = #


	# 	SubForms = {}
	# 	FreqSets = {}
	# 	# = = = [ Indexes between authors and pubs ] = = = #
	# 	# Per each pub p:
	# 	#   1. A2Collabs: Index between author_i and collaboration_p.
	# 	for p in P2Teams:

	# 		similar_teams = list(P2Teams[p].keys())
	# 		union =  {}
	# 		for t in similar_teams:
	# 			for single in t.split("_"):
	# 				if single not in union:
	# 					union[single] = True
	# 		union = "_".join(  map( str , sorted(map( int , union.keys() )) ) )

	# 		# union_pubs = None
	# 		# if union in A2Pubs:
	# 		# 	union_pubs = sorted(A2Pubs[union])
	# 		# # print( similar_teams ," : ", union, " -> ",union_pubs )

	# 		if union not in FreqSets:
	# 			FreqSets[union] = {}

	# 		for t in similar_teams:
	# 			# print( "\t" , t )
	# 			collabs = list(Cores[t].keys())
	# 			# print( "\t\t" , collabs )
	# 			for collab in collabs:
	# 				FreqSets[union][collab] = True
	# 				SubForms[collab] = True
	# 				# print( "\t\t" , A2Pubs[collab] )
	# 			# print("")
	# 		# print("\t- - - - - - ")
	# 		# print( "" )
	# 		# print( "" )






	# 	weighted_cliques = []

	# 	for i in A2Pubs:
	# 		if i in FreqSets or i in SubForms:
	# 			pass
	# 		else:
	# 			nb_pubs = len(A2Pubs[i])
	# 			# print("not grouped:"," ["+str(nb_pubs)+"] ", i)
	# 			if len(i.split( "_" )) > 3:
	# 				weighted_cliques.append( [nb_pubs , sorted(A2Pubs[i]) , i] )

	# 	for itemset in FreqSets:
	# 		pubs = {}
	# 		# print( itemset )
	# 		# print( "\t", list(FreqSets[itemset].keys()) )
	# 		for i in FreqSets[itemset]:
	# 			# print( "\t" , i)
	# 			for p in A2Pubs[i]:
	# 				pubs[p] = True
	# 		nb_pubs = len(pubs.keys())
	# 		# print( "yes grouped:" , " ["+str(nb_pubs)+"] " , itemset )
	# 		if len(itemset.split( "_" )) > 3:
	# 			weighted_cliques.append( [nb_pubs , sorted(list(pubs.keys())) , itemset] )
	# 		# print( "\t" ,  )
	# 		# print("")
	# 		# print("")

	# 	weighted_cliques = sorted(weighted_cliques, key = lambda x: x[0] , reverse=True)

	# 	print( "P:" , P )
	# 	print( "A:" , len(A2Collabs.keys()) )
	# 	print( "C:" , len( weighted_cliques )  )
	# 	print("")

	# 	Nodes = {}
	# 	for clique in weighted_cliques:
	# 		if clique[0] > 1:
	# 			print( clique[0] ,"\t", clique[2] )
	# 			for node in clique[2].split("_"):
	# 				author = int(node)
	# 				if author not in Nodes:
	# 					Nodes[author] = 0
	# 				Nodes[author] += 1

	# 			# print( "\t" , clique[1] ) 
	# 			# print("")

	# 	# for i in Nodes:
	# 	# 	if Nodes[i]>1:
	# 	# 		print( Nodes[i] ,"\t", i )


	# 	connDBLP.close()
	# 	self.stats["times"].append( [ self.get_time_s( start ) , "sqlite extr + elems-OCCs" ] )
	# 	return True


	# def calc_ACM_OCCs_old ( self , mainfolder ,timeunit="2016" ):
	# 	# # [ = = = = = = = [ COOCs CALCULATION ] = = = = = = = ] # # 
	# 	if self.DEBUG: print( time.strftime("%H:%M:%S") , "calc_ACM_OCCs" )

	# 	self.stats["times"].append( [ time.strftime("%H:%M:%S") , "calc_ACM_OCCs" ] )

	# 	start = time.time()
		
	# 	N_aid = 0
	# 	N_np = 0
	# 	nID = 0
	# 	query = 'find "'+mainfolder+'"/  -type f -name "'+timeunit+'*"'
	# 	# print( query )

	# 	FreqD = {}

	# 	pubs_folder = os.popen(  query  )
	# 	for i in pubs_folder.readlines():

	# 		# if (N_np % 5000)==0:
	# 		# 	if N_np==0: print( "\t","cest parti!" )
	# 		# 	else: print("\t",N_np,"pubs analized.")

	# 		pub = i.replace("\n","")
	# 		json_data = open( pub , "r")
	# 		pubmetadata = jsonold.load(json_data)
	# 		N_np += 1

	# 		if "authors" in pubmetadata and "id" in pubmetadata["authors"][0]:				
	# 			authors_ids = []
	# 			for a in pubmetadata["authors"]:
	# 				# sID = int(a["id"])
	# 				# if sID not in self.ACM:
	# 				# 	nID += 1
	# 				# 	self.ACM[sID] = { "id" : nID , "n": a["name"] }
	# 				# nID = self.ACM[sID]["id"]
	# 				nID = int(a["id"])
	# 				authors_ids.append( nID )
	# 				if nID not in FreqD:
	# 					FreqD[ nID ] = 0
	# 				FreqD[ nID ] += 1

	# 			self.rawdata.append( authors_ids )
	# 			N_aid += 1

	# 	self.stats["times"].append( [ self.get_time_s( start ) , "data-extr + elems-OCCs" ] )
	# 	self.stats["Pubs"] = N_np
	# 	self.stats["Pubs+Aid"] = N_aid
	# 	self.occurrences = FreqD

